@@ -20,10 +20,8 @@
 # that computes the covariance matrix of the estimate produced by Ipfp.
 # ------------------------------------------------------------------------------
 
-library(cmm)
-
 Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000, 
-                 tol = 1e-10, na.target = FALSE, compute.cov = FALSE, ...) {
+                 tol = 1e-10, na.target = FALSE) {
   # Update an array using the iterative proportional fitting procedure.
   #
   # Author: J. Barthelemy
@@ -33,7 +31,7 @@ Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000,
   #         be non-negative.
   #   target.list: A list of the target margins provided in target.data. Each
   #                component of the list is an array whose cells indicates
-  #                 which dimension the corresponding margin relates to.
+  #                which dimension the corresponding margin relates to.
   #   target.data: A list containing the data of the target margins. Each
   #                component of the list is an array storing a margin.
   #                The list order must follow the one defined in target.list. 
@@ -47,19 +45,16 @@ Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000,
   #        (stopping criterion); must be greater than 0.
   #   na.target: If set to TRUE, allows the targets to have NA cells. In that
   #              case the margins consistency is not checked.
-  #   compute.cov: If set to TRUE, then the function also return the covariance
-  #                matrices of the updated cells and cells proportion.
-  #   ...: Additional arguments that can be passed to the IpfpCovar function
-  #        if compute.cov = TRUE. See IpfpCovar documentation.
   #
-  # Returns: A list generated whose elements are
-  #   xi.hat: An array of the same dimension of seed whose margins match the
-  #           ones specified in target.list.
+  # Returns: A mipfp object consisting of a list whose elements are
+  #   call: A call object in which all the specified arguments are given by
+  #         their full names.
+  #   method: The selected method for estimation.
   #   stp.crit: The final value of the stopping criterion.
   #   evol.stp.crit: Evolution of the stopping criterion over the iterations.
   #   conv: A boolean indicating whether the algorithm converged to a solution.
-  #   check.margins: A list returning, for each margin, the absolute maximum 
-  #                 deviation between the desired and generated margin.
+  #   error.margins: A list returning, for each margin, the absolute maximum 
+  #                  deviation between the target and generated margin.
   
   # checking if a seed is provided
   if (is.null(seed) == TRUE) {
@@ -92,19 +87,19 @@ Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000,
   }
   
   # checking if NA allowed and requesting the covariance matrices
-  if (na.target == TRUE & compute.cov == TRUE) {
+  if (na.target == TRUE) { #& compute.cov == TRUE) {
     warning('Missing values allowed in the target margins.
              Computation of the covariance matrices set to FALSE!')
     compute.cov <- FALSE
   }
   
   # checking the margins consistency if no missing values in the targets
-  check.margins <- TRUE  
+  error.margins <- TRUE  
   if (na.target == FALSE) {
     if (length(target.data) > 1) {
       for (m in 2:length(target.data)) {      
         if (abs(sum(target.data[[m-1]]) - sum(target.data[[m]])) > 1e-10) {
-          check.margins <- FALSE
+          error.margins <- FALSE
           warning('Target not consistents - shifting to probabilities!
                   Check input data!\n')
           break
@@ -119,14 +114,14 @@ Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000,
   }
   
   # if margins are not consistent, shifting from frequencies to probabilities
-  if (check.margins == FALSE) {
+  if (error.margins == FALSE) {
     seed <- seed / sum(seed)
     for (m in 1:length(target.data)) {
       target.data[[m]] <- target.data[[m]] / sum(target.data[[m]])
     }
   }
   
-  if (print == TRUE & check.margins == TRUE & na.target == FALSE) {
+  if (print == TRUE & error.margins == TRUE & na.target == FALSE) {
     cat('Margins consistency checked!\n')
   }
     
@@ -138,7 +133,7 @@ Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000,
   # ipfp iterations
   for (i in 1:iter) {
     
-    if (print) {
+    if (print == TRUE) {
       cat('... ITER', i, '\n')
     } 
     
@@ -147,7 +142,7 @@ Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000,
             
     # loop over the constraints
     for (j in 1:length(target.list)) {
-      # ... extracting current margins
+      # ... extracting current margins      
       temp.sum <- apply(result, target.list[[j]], sum)
       # ... computation of the update factor, taking care of 0 and NA cells   
       update.factor <- ifelse(target.data[[j]] == 0 | temp.sum == 0, 0,
@@ -156,7 +151,7 @@ Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000,
         update.factor[is.na(update.factor)] <- 1;
       }
       # ... apply the update factor
-      result <- sweep(result,target.list[[j]], update.factor, FUN = "*")
+      result <- sweep(result, target.list[[j]], update.factor, FUN = "*")
     }
     
     # stopping criterion
@@ -164,13 +159,14 @@ Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000,
     tmp.evol.stp.crit[i] <- stp.crit
     if (stp.crit < tol) {
       converged <- TRUE
-      if (print) {
+      if (print == TRUE) {
+        cat('       stoping criterion:', stp.crit, '\n')
         cat('Convergence reached after', i, 'iterations!\n')
       } 
       break
     }
     
-    if (print) {
+    if (print == TRUE) {
       cat('       stoping criterion:', stp.crit, '\n')
     }
     
@@ -188,7 +184,11 @@ Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000,
   if (na.target == FALSE) {
     for (j in 1:length(target.list)) {
       diff.margins[j] = max(abs(target.data[[j]] 
-                                - apply(result, target.list[[j]], sum))) 
+                                - apply(result, target.list[[j]], sum)))
+      if (is.null(names(dimnames(seed))) == FALSE) {
+        names(diff.margins)[j] <- paste(names(dimnames(seed))[target.list[[j]]],
+                                        collapse = ".")
+      }
     }
   }
   
@@ -200,83 +200,19 @@ Ipfp <- function(seed, target.list, target.data, print = FALSE, iter = 1000,
   
   # gathering the results in a list
   results.list <- list("x.hat" = result, "p.hat" = result.prop, 
-                      "stp.crit" = stp.crit, "conv" = converged, 
-                      "check.margins" = diff.margins, 
-                      "evol.stp.crit" = evol.stp.crit);
+                       "conv" = converged, "error.margins" = diff.margins, 
+                       "evol.stp.crit" = evol.stp.crit)
   
-  # adding covariance if requested
-  if (compute.cov == TRUE) {
-    results.list$p.hat.cov <- IpfpCov(result, seed, target.list, ...)
-    results.list$x.hat.cov <- results.list$p.hat.cov * sum(result)^2
-    results.list$p.hat.se <- sqrt(diag(results.list$p.hat.cov))
-    results.list$x.hat.se <- sqrt(diag(results.list$x.hat.cov))
-  }
+  # adding the method applied
+  results.list$method <- "ipfp"
+  
+  # adding the calling expression
+  results.list$call <- match.call()
+  
+  # updating the class attribute
+  class(results.list) <- c("list", "mipfp")      
   
   # returning the result
   return(results.list)
   
 }
-
-IpfpCov <- function(estimate, seed, target.list, replace.zeros = 1e-10) {
-  # Compute the covariance matrix of the estimators produced by Ipfp.
-  #
-  # This function determines the covariance matrix of the estimated proportions
-  # using the Delta method given in the paper "Models for Contingency Tables 
-  # With Known Margins When Target and Sampled Populations Differ" written by 
-  # Little and Wu (1991).
-  #
-  # Author: J. Barthelemy
-  #
-  # Args:
-  #   estimate: The array of estimate produced by the Ipfp function.
-  #   seed: The initial multi-dimensional array updated by Ipfp. 
-  #   target.list: A list of the target margins used by the Ipfp function. Each
-  #                component of the list is an array whose cells indicates
-  #                which dimension the corresponding margin relates to.
-  #   replace.zeros: If 0-cells are to be found in either the seed or the
-  #                  estimate arrays, then their values are replaced with this
-  #                  value.
-  #
-  # Returns: A covariance matrix of the estimated probabilities (last index move
-  #          fastest).
-    
-  n <- sum(seed)  
-  seed.prob <- Array2Vector(seed / sum(seed))
-  estimate.prob <- Array2Vector(estimate / sum(estimate))
-  
-  # checking if 0-cells values and replace them with a small value  
-  seed.prob <- ifelse(seed.prob == 0, replace.zeros, seed.prob)    
-  estimate.prob <- ifelse(estimate.prob == 0, replace.zeros, estimate.prob)
-  
-  # computation of the diagonal matrix filled with the inverses of seed and
-  # estimated probabilities
-  D.seed <- diag(1 / seed.prob)
-  D.estimate <- diag(1 / estimate.prob)
-  
-  # computation of A such that A' * vector(estimate) = vector(target.data)
-  # ... one line filled with ones
-  A.transp <- matrix(1, nrow = 1, ncol = length(estimate.prob))  
-  # ... constrainst (removing the first one since it is redundant information)
-  for (j in 1:length(target.list)) {
-    marg.mat <- cmm::MarginalMatrix(var = 1:length(dim(seed)), 
-                                    marg = target.list[[j]], 
-                                    dim = dim(seed))[-1,]
-    A.transp <- rbind(marg.mat, A.transp, deparse.level = FALSE)
-  }  
-  A <- t(A.transp)
-  
-  # removing the linearly dependant columns from A (redundant constrainst)
-  A <- GetLinInd(A)$mat.li
-    
-  # computation of the orthogonal complement of A (using QR decomposition)
-  K <- qr.Q(qr(A), complete = TRUE)[, (dim(A)[2] + 1):dim(A)[1]]
-
-  # computation of the variance  
-  estimate.var <- (1 / n) * K %*% solve((t(K) %*% D.estimate %*% K)) %*%
-                  t(K) %*% D.seed %*% K %*%
-                  solve(t(K) %*% D.estimate %*% K) %*% t(K)   
-
-  # returning the result
-  return(estimate.var)
-  
-}
